@@ -2,6 +2,8 @@
 import imaplib
 import email
 import getpass
+
+from transparentpath import TransparentPath
 import chardet
 import sys
 import traceback
@@ -12,13 +14,22 @@ from time import sleep, time
 from pathlib import Path
 import numpy as np
 from typing import Union, List, Dict, Optional
-from datetime import datetime
+from datetime import datetime, date as ddate
 from multiprocessing.pool import ThreadPool
 import logging
 
+from .mailsender import MailSender
+
 logger = logging.getLogger(__name__)
 
 
+def login(username, password, server, port):
+    mail = imaplib.IMAP4_SSL(server, port)
+    mail.login(username, password)
+    mail.select()
+    return mail
+
+
 class MailException(Exception):
     """Any kind of mail error"""
 
@@ -44,8 +55,8 @@ def get_datetime_now(fmt: str = "%d%m%Y %H:%M:%S") -> str:
 
     """
 
-    date = datetime.now()
-    return date.strftime(fmt)
+    d = datetime.now()
+    return d.strftime(fmt)
 
 
 # noinspection PyBroadException, PyUnresolvedReferences
@@ -227,6 +238,8 @@ class MailMonitor(object):
         connect: bool = False,
         overwrite: bool = True,
         max_threads: int = None,
+        send_errors_to: str = None
+
     ):
 
         if username is None:
@@ -236,6 +249,7 @@ class MailMonitor(object):
         if "@" not in username:
             username = f"{username}@{MailMonitor.default_mail}"
 
+        self.send_errors_to = send_errors_to
         self.username = username
         self.token = token
         self.port = port
@@ -260,10 +274,7 @@ class MailMonitor(object):
         while True:
             attempts += 1
             try:
-                self.mailbox = imaplib.IMAP4_SSL(self.hostname, self.port)
-                # Login to your account
-                self.mailbox.login(self.username, self.token)
-                self.mailbox.select()
+                self.mailbox = login(self.username, self.token, self.hostname, self.port)
                 if talk:
                     logger.info("...successful")
                 break
@@ -276,7 +287,7 @@ class MailMonitor(object):
                     sleep(1)
 
     @staticmethod
-    def configure_date(date: Union[str, datetime]) -> str:
+    def configure_date(d: Union[str, datetime]) -> str:
         """
         Converts datetime format to mail date format
 
@@ -284,7 +295,7 @@ class MailMonitor(object):
 
         Parameters
         ----------
-        date : Union[str, datetime]
+        d : Union[str, datetime]
             date to convert
 
         Returns
@@ -292,8 +303,8 @@ class MailMonitor(object):
         str
             converted date
         """
-        if isinstance(date, str):
-            date = datetime.strptime(date, "%Y-%m-%d")
+        if isinstance(d, str):
+            d = datetime.strptime(d, "%Y-%m-%d")
         m_dict = {
             "01": "Jan",
             "02": "Feb",
@@ -308,15 +319,15 @@ class MailMonitor(object):
             "11": "Nov",
             "12": "Dec",
         }
-        if len(str(date.month)) == 1:
-            m = "0" + str(date.month)
+        if len(str(d.month)) == 1:
+            m = "0" + str(d.month)
         else:
-            m = str(date.month)
+            m = str(d.month)
         mv = m_dict[m]
-        return str(date.day) + "-" + mv + "-" + str(date.year)
+        return str(d.day) + "-" + mv + "-" + str(d.year)
 
     @staticmethod
-    def read_date(date: str) -> datetime.date:
+    def read_date(d: str) -> datetime.date:
         """
         convert date from mail format for datetime
 
@@ -324,7 +335,7 @@ class MailMonitor(object):
 
         Parameters
         ----------
-        date : str
+        d : str
             mail format
 
         Returns
@@ -345,18 +356,18 @@ class MailMonitor(object):
             "11": "Nov",
             "12": "Dec",
         }
-        day = date[:2]
+        day = d[:2]
         m_dict = {i: j for i, j in zip(m_dict.values(), m_dict.keys())}
-        month = m_dict[date[3:6]]
-        year = date[7:11]
-        date = datetime.strptime(year + "-" + month + "-" + day, "%Y-%m-%d")
-        return date.date()
+        month = m_dict[d[3:6]]
+        year = d[7:11]
+        d = datetime.strptime(year + "-" + month + "-" + day, "%Y-%m-%d")
+        return d.date()
 
     # noinspection PyUnresolvedReferences
     def monitor(
         self,
         conditions: Union[dict, List[dict]],
-        to_path: Union[Union["TransparentPath", Path, str], List[Union["TransparentPath", Path, str]]],
+        to_path: Union[Union[TransparentPath, Path, str], List[Union[TransparentPath, Path, str]]],
         time_to_sleep: Union[int, List[int]] = 60,
         mailbox: Union[str, List[str]] = "INBOX",
         overwrite: Union[bool, List[bool]] = None,
@@ -443,14 +454,15 @@ class MailMonitor(object):
 
     def fetch_one_mail(
         self,
-        save_dir: Union[str, Path],
+        save_dir: Union[str, Path, TransparentPath],
         state: Optional[str] = "ALL",
         subject: Optional[str] = None,
         sender: Optional[str] = None,
         body: Optional[str] = None,
-        date: Union[str, datetime, None] = None,
+        date: Union[str, datetime, ddate, None] = None,
         mailbox: Union[str, List[str]] = "INBOX",
-        modes: Dict[str, str] = None
+        modes: Dict[str, str] = None,
+        expected_attachments: Union[int, bool] = True
     ) -> Union[bool, tuple]:
         if modes is None:
             modes = {"start": "exact", "end": "exact"}
@@ -463,7 +475,8 @@ class MailMonitor(object):
             start_date=date,
             end_date=date,
             mailbox=mailbox,
-            modes=modes
+            modes=modes,
+            expected_attachments=expected_attachments
         )
 
     def fetch_attachment(self, uid: str) -> dict:
@@ -492,17 +505,18 @@ class MailMonitor(object):
         return ret
 
     def fetch_mails(
-            self,
-            save_dir: Union[str, Path],
-            state: Optional[str] = "ALL",
-            subject: Optional[str] = None,
-            sender: Optional[str] = None,
-            body: Optional[str] = None,
-            start_date: Union[str, datetime, None] = None,
-            end_date: Union[str, datetime, None] = None,
-            mailbox: Union[str, List[str]] = "INBOX",
-            modes: Dict[str, str] = None,
-            duplicated: str = "last"
+        self,
+        save_dir: Union[str, Path, TransparentPath],
+        state: Optional[str] = "ALL",
+        subject: Optional[str] = None,
+        sender: Optional[str] = None,
+        body: Optional[str] = None,
+        start_date: Union[str, datetime, None] = None,
+        end_date: Union[str, datetime, None] = None,
+        mailbox: Union[str, List[str]] = "INBOX",
+        modes: Dict[str, str] = None,
+        duplicated: str = "last",
+        expected_attachments: Union[int, bool] = True
     ):
         """
         Will fetch to attachments of mails based on the dates on arrival and the select modes
@@ -521,6 +535,9 @@ class MailMonitor(object):
             keys are 'start' and 'end', values can be 'exact', 'nearest', 'next', 'last'
         duplicated : str = "last" ("first", "last", "all")
             What to do if there are several mails the same day
+        expected_attachments: Union[int, bool]
+            Can be True to tell the function that it should raise an error if no attachment is found in the mail. Can
+            also be an integer to tell the exact number of expected attachments (Default value = False)
 
         Returns
         -------
@@ -599,31 +616,44 @@ class MailMonitor(object):
         good_dates = [d for d in dates if (d >= best_start_date) and (d <= best_end_date)]
         good_dates.sort()
 
-        bp = Path(save_dir)
+        if not isinstance(save_dir, TransparentPath):
+            bp = TransparentPath(save_dir)
+        else:
+            bp = save_dir
         if not bp.exists():
             bp.mkdir()
 
         def save_one(path, uid_):
             if not path.exists():
                 path.mkdir()
+            attachments = 0
             for i_, j_ in self.fetch_attachment(uid_).items():
                 (path / i_).write_bytes(j_)
+                attachments += 1
 
-        for date in good_dates:
-            if len(dict_date[date]) > 1:
+            if isinstance(expected_attachments, bool):
+                if expected_attachments is True and attachments == 0:
+                    logger.warning("Expected attachments in the mail, but found none.")
+                    return False, None
+            elif isinstance(expected_attachments, int) and attachments != expected_attachments:
+                logger.warning(f"Expected {expected_attachments} attachments in the mail, but found {attachments}.")
+                return False, None
+
+        for d in good_dates:
+            if len(dict_date[d]) > 1:
                 if duplicated == "last":
-                    uid = [dict_date[date][-1]]
+                    uid = [dict_date[d][-1]]
                 elif duplicated == "first":
-                    uid = [dict_date[date][0]]
+                    uid = [dict_date[d][0]]
                 else:
-                    uid = [dict_date[date]]
+                    uid = [dict_date[d]]
             else:
-                uid = [dict_date[date][0]]
+                uid = [dict_date[d][0]]
 
             bps = bp
             if len(good_dates) > 1:
-                logger.info(f"Found {len(uid)} mail(s) on {date}")
-                bps = bp / date.strftime('%Y-%m-%d')
+                logger.info(f"Found {len(uid)} mail(s) on {d}")
+                bps = bp / d.strftime('%Y-%m-%d')
             if len(uid) > 1:
                 for i in range(len(uid)):
                     save_one(bps.append(f"_{i}"), uid[i])
@@ -777,24 +807,25 @@ class MailMonitor(object):
 
         the_while(self, timeout)
 
-    # noinspection PyUnresolvedReferences
-    def send(self, msg):
-        new_message = email.message.Message()
-        new_message["From"] = f"{self.username}@{MailMonitor.default_mail}"
-        new_message["Subject"] = "MailMonitoring ended with Exception"
-        new_message.set_payload(msg)
-        self.open_connection()
-        self.mailbox.append(
-            "INBOX",
-            "",
-            imaplib.Time2Internaldate(time()),
-            str(new_message).encode(),
-        )
-        if not self.mailbox.state == "LOGOUT":
-            self.mailbox.select()
-            self.mailbox.close()
-            self.mailbox.logout()
-        logger.info(f"Sent warning message to {self.username}@{MailMonitor.default_mail}")
+    def send(self, msg, to: str = None):
+        attempts = 0
+        while attempts < 2:
+            # noinspection PyBroadException
+            try:
+                mailsender = MailSender(sender=self.username, passwd=self.token)
+                if to is None:
+                    mailsender.send_mail(
+                        adresses=self.username, subject="MailMonitoring ended with Exception", text=msg
+                    )
+                    logger.info(f"Sent warning message to {self.username}")
+                else:
+                    mailsender.send_mail(adresses=to, subject="MailMonitoring ended with Exception", text=msg)
+                    logger.info(f"Sent warning message to {to}")
+                break
+            except self.mailbox.abort:
+                logger.warning("imaplib.abort exception raised. Attempting to reconnect.")
+                self.mailbox = login(self.username, self.token, self.hostname, self.port)
+                attempts += 1
 
 
 _excepthook = getattr(sys, "excepthook")
@@ -821,7 +852,7 @@ def overload_raise(ex, val, tb):
         li = traceback.format_exception(ex, val, tb)
         to_send = "".join(li)
         for mm in MailMonitor.instances:
-            mm.send(to_send)
+            mm.send(to_send, to=mm.send_errors_to)
         _excepthook(ex, val, tb)
     else:
         sys.exit(0)
